
using System.Runtime.Serialization;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using System.Drawing;
using System.Collections;
using Syncfusion.Grouping;


namespace GGC_RefreshGrouping
{    
    public class CustomCellModel : GridTextBoxCellModel
    {
        protected CustomCellModel(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }

        public CustomCellModel(GridModel grid)
            : base(grid)
        {
        }

        public override GridCellRendererBase CreateRenderer(GridControlBase control)
        {
            return new CustomCellRenderer(control, this);
        }

        //used to provide optimal height and width for cell (as in grid.ColWidths.ResizeToFit calls)
        protected override Size OnQueryPrefferedClientSize(Graphics g, int rowIndex, int colIndex, GridStyleInfo style, GridQueryBounds queryBounds)
        {
            Size sz = base.OnQueryPrefferedClientSize(g, rowIndex, colIndex, style, queryBounds);

            //adjust to allow for button width
            sz.Width = sz.Width + ButtonBarSize.Width;

            return sz;
        }
    }
        
    public class CustomCellRenderer : GridTextBoxCellRenderer
    {
        #region Data Members
        
        private IValue m_OldValue;               
                
        #endregion // Data Members

        #region Constructor

        public CustomCellRenderer(GridControlBase grid, GridCellModelBase cellModel)
            : base(grid, cellModel)
        {
            
        }

        #endregion

        #region Overrides        
        
        private ArrayList groups;
        protected override bool OnSaveChanges()
        {   
            m_OldValue.Parse(TextBox.Text);
            this.ControlValue = m_OldValue;
             bool bvalue = base.OnSaveChanges();
            GridTableControl tc = this.Grid as GridTableControl;
            if (tc != null)
            {
                groups = new ArrayList();
                iterate(tc.Table.Engine.GroupingControl.Table.TopLevelGroup);
                tc.Reinitialize(); 
                iterateExpand(tc.Table.Engine.GroupingControl.Table.TopLevelGroup); 
            }
            return bvalue;
        }
        
        public void iterate(Group g)
        {
            if (g.IsExpanded )
                groups.Add(g.Name + "_" + System.Convert.ToString(g.Category));
            foreach (Group gr in g.Groups)
                iterate(gr);
        }
        protected override void OnInitialize(int rowIndex, int colIndex)
        {
            base.OnInitialize(rowIndex, colIndex);
            m_OldValue = ControlValue as IValue;            
        }

        public void iterateExpand(Group g)
        {
            try
            {
                if (g != null)
                {
                    string key = g.Name + "_" + System.Convert.ToString(g.Category);
                    if (groups.IndexOf(key) > -1 && !g.IsExpanded)
                        g.IsExpanded = true;

                    foreach (Group gr in g.Groups)
                        iterateExpand(gr);
                }
            }
            catch (System.Exception ex )
            {
                System.Console.WriteLine(ex.StackTrace); 
            }
        }

        #endregion       
    }
}